import os

from typing import Callable, Iterable, Generator


__version__ = "0.0.1"


def changeextension_functor(
    newext: str,
    dotfile: bool = False,
) -> Callable[[str], str]:
    def func(audiofile: str) -> str:
        path, fne = os.path.split(audiofile)
        fn, ext = os.path.splitext(fne)
        if dotfile:
            o = os.path.join(path, "." + fn + "." + newext)
        else:
            o = os.path.join(path, fn + "." + newext)
        return o

    return func


getmoodfile = changeextension_functor("mood", dotfile=True)
getnfofile = changeextension_functor("nfo", dotfile=False)
getbadmoodfile = changeextension_functor("badmood", dotfile=True)


def setbadmoodfile(audio: str) -> None:
    badmoodfile = getbadmoodfile(audio)
    with open(badmoodfile, "w") as f:
        f.write(
            "This file is damaged and its mood cannot be computed",
        )


def unsetbadmoodfile(audio: str) -> None:
    badmoodfile = getbadmoodfile(audio)
    os.unlink(badmoodfile)


def audio_to_audioandmood(
    seq: Iterable[str],
) -> Generator[tuple[str, str], None, None]:
    for s in seq:
        yield (s, getmoodfile(s))


def absolutize_paths(seq: Iterable[str]) -> Generator[str, None, None]:
    for s in seq:
        yield os.path.abspath(s)


def video_to_videoandnfo(
    seq: Iterable[str],
) -> Generator[tuple[str, str], None, None]:
    for s in seq:
        yield (s, getnfofile(s))


def walk_tree(
    path: str,
) -> Generator[str, None, None]:
    for base, dirs, files in os.walk(path):
        for f in files:
            yield os.path.join(base, f)


def filter_audio(
    seq: Iterable[str],
) -> Generator[str, None, None]:
    for s in seq:
        p, e = os.path.splitext(s)
        if e.lower() in AUDIO_EXTS:
            yield s


AUDIO_EXTS = [".mp3", ".ogg", ".flac", ".wma", ".opus"]

VIDEO_EXTS = (
    ".m4v .3g2 .3gp .nsv .tp .ts .ty .strm .pls .rm .rmvb .mpd .ifo .mov .qt "
    ".divx .xvid .bivx .vob .nrg .pva .wmv .asf .asx .ogm .m2v .avi .mpg "
    ".mpeg .mp4 .mkv .mk3d .avc .vp3 "
    ".svq3 .nuv .viv .dv .fli .flv .wpl .vdr .dvr-ms .xsp .mts .m2t "
    ".m2ts .evo .ogv .sdp .avs "
    ".rec .pxml .vc1 .h264 .rcv .webm .bdmv .wtv "
    ".mp4 .flv .avi .m4v .mpg .mkv .webm"
).split()


def filter_video(
    seq: Iterable[str],
) -> Generator[str, None, None]:
    for s in seq:
        p, e = os.path.splitext(s)
        if e.lower() in VIDEO_EXTS:
            yield s


def filter_inaccessible(
    seq: Iterable[str],
) -> Generator[str, None, None]:
    for s in seq:
        if os.access(s, os.R_OK) and os.access(
            os.path.dirname(s),
            os.W_OK | os.X_OK,
        ):
            yield s


def filter_nonexistentnfo(
    seq: Iterable[str],
) -> Generator[str, None, None]:
    for video in seq:
        nfofile = getnfofile(video)
        if not os.path.exists(nfofile):
            yield video
        else:
            videofile_time = os.stat(video).st_mtime
            nfofile_time = os.stat(nfofile).st_mtime
            if videofile_time > nfofile_time:
                os.unlink(nfofile)
                yield video


def filter_nonexistentmood(
    seq: Iterable[str],
) -> Generator[str, None, None]:
    for audio in seq:
        moodfile = getmoodfile(audio)
        if not os.path.exists(moodfile):
            yield audio
        else:
            audiofile_time = os.stat(audio).st_mtime
            moodfile_time = os.stat(moodfile).st_mtime
            if audiofile_time > moodfile_time:
                os.unlink(moodfile)
                yield audio


def filter_badmood(
    seq: Iterable[str],
) -> Generator[str, None, None]:
    for audio in seq:
        badmoodfile = getbadmoodfile(audio)
        if not os.path.exists(badmoodfile):
            yield audio
        else:
            audiofile_time = os.stat(audio).st_mtime
            badmoodfile_time = os.stat(badmoodfile).st_mtime
            if audiofile_time > badmoodfile_time:
                unsetbadmoodfile(audio)
                yield audio
