import os
import sys
import subprocess
import mediacollectiongrooming

import typing


def domoodbar(file: str, ofn: str) -> str:
    return subprocess.check_output(
        ["nice", "-n20", "ionice", "-c3", "moodbar", "-o", ofn, file],
        stderr=subprocess.STDOUT,
        universal_newlines=True,
    )


def find_songs(path: str) -> typing.Generator[tuple[str, str], None, None]:
    a = mediacollectiongrooming.walk_tree(path)
    b = mediacollectiongrooming.filter_audio(a)
    c = mediacollectiongrooming.filter_inaccessible(b)
    d = mediacollectiongrooming.filter_badmood(c)
    e = mediacollectiongrooming.filter_nonexistentmood(d)
    f = mediacollectiongrooming.audio_to_audioandmood(e)
    return f


E_NOMOODBAR = "no moodbar"
E_NOCODEC = "no codec"
E_INTR = "interrupted"


def thread_main(audio: str, mood: str) -> str | None:
    try:
        domoodbar(audio, mood)
    except KeyboardInterrupt:
        if os.path.exists(mood):
            os.unlink(mood)
        return E_INTR
    except subprocess.CalledProcessError as e:
        if e.returncode == 127:
            return E_NOMOODBAR
        elif e.returncode == 2:
            if "Bus error" in e.output:
                print(
                    "Bad audio file processing",
                    audio,
                    "\n",
                    "    ->",
                    e.output,
                    file=sys.stderr,
                )
                if os.path.exists(mood):
                    os.unlink(mood)
                mediacollectiongrooming.setbadmoodfile(audio)
            else:
                print(e.output, file=sys.stderr)
                return E_NOCODEC
        else:
            print(
                sys.stderr,
                "Return code %s processing" % e.returncode,
                audio,
                "\n",
                "    ->",
                e.output,
                file=sys.stderr,
            )
            if os.path.exists(mood):
                os.unlink(mood)
    except BaseException as e:
        print("Failure processing", audio, "\n", "    ->", e, file=sys.stderr)
        if os.path.exists(mood):
            os.unlink(mood)

    return None


def main() -> int:
    basedir = sys.argv[1]
    iterable = find_songs(basedir)
    for song, moodfile in iterable:
        ret = thread_main(song, moodfile)
        if ret == E_NOMOODBAR:
            print(
                "error: no moodbar program available",
                file=sys.stderr,
            )
            return 16
        if ret == E_NOCODEC:
            print(
                "error: moodbar program cannot decode this file",
                file=sys.stderr,
            )
        if ret == E_INTR:
            print(
                "interrupted",
                file=sys.stderr,
            )
            return 4
    return 0


if __name__ == "__main__":
    sys.exit(main())
